#include <iostream>
#include <string.h>
#include <string>
#include <windows.h>

#include "pe.h"
#include "hyperion.h"
#include "fileaccess.h"
#include "peanalysis.h"
#include "createoutput.h"

using namespace std;
using namespace hyperion;

int main(int argc, char *argv[])
{
    if(argc<3){
        cout<<"Hyperion PECrypter 1.1 by Christian Ammann"<<endl;
        cout<<"Usage: crypter.exe <infile> <outfile>"<<endl;
        return 0;
    }

    //input file
    const char* infile_name = argv[1];
    //output file
    const char* output_name = argv[2];

    //open input file
	cout<<endl;
    FileInMemory* infile = new FileInMemory(infile_name);
    if(!infile->open) return 0;

    //collect data from the input file's pe header
    PE32Analysis* infile_analysis = new PE32Analysis(infile);
    if(infile_analysis->error){
        delete infile_analysis;
        delete infile;
        return 0;
    }
    cout << endl;

    //create opcodes which write the input file into memory
    if(!createFasmInfileLoader(infile)){
        delete infile_analysis;
        delete infile;
        return 0;
    }

    //create image base asm representation and write it to IMAGE_BASE_INC
    if(!createFasmInclude(IMAGE_BASE_INC, IMAGE_BASE_LABEL, *(infile_analysis->image_base))){
        delete infile_analysis;
        delete infile;
        return 0;
    }

    //create image size asm representation and write it to IMAGE_SIZE_INC
    if(!createFasmInclude(IMAGE_SIZE_INC, IMAGE_SIZE_LABEL, *(infile_analysis->size_of_image))){
        delete infile_analysis;
        delete infile;
        return 0;
    }

    //create an keysize equate for fasm sources
    if(!createFasmKeySize()){
        delete infile_analysis;
        delete infile;
        return 0;
    }

    //get current directory
    char* current_directory = new char[3000];
    if(!GetCurrentDirectory(3000, current_directory)){
        cout<<"Error: Could not receive current directory";
        delete infile_analysis;
        delete infile;
        return 0;
    }

    //start fasm to generate a packed executable
    string cmd_line = "";
    cmd_line+=FASM_EXECUTABLE;
    cmd_line+=" ";
    cmd_line+=CONTAINER_MAIN;
    cmd_line+=" ";
    cmd_line+=output_name;
    char* cmd_line_noconst = new char[cmd_line.size()+1];
    strcpy(cmd_line_noconst, cmd_line.c_str());
    STARTUPINFO startup_info;
    ZeroMemory( &startup_info, sizeof(startup_info) );
    startup_info.cb = sizeof(startup_info);
    PROCESS_INFORMATION process_info;
    cout<<"Starting FASM with the following parameters:"<<endl;
    cout<<"Commandline: "<<cmd_line_noconst<<endl;
    cout<<"FASM Working Directory: "<<current_directory<<endl<<endl;
    if(CreateProcess(FASM_EXECUTABLE, cmd_line_noconst, 0, 0, false, 0, 0, current_directory, &startup_info, &process_info))
        cout<<"Executing fasm.exe"<<endl;
    else{
        cout<<"Error: Could not start fasm.exe"<<endl;
        cout<<"Error Code: "<<GetLastError();
    }

    //close application
    cout << endl;
    delete []cmd_line_noconst;
    delete []current_directory;
    delete infile_analysis;
    delete infile;
    return 0;
}
